#!/usr/bin/python
# -*- coding: UTF-8 -*-


from flask import Flask
import pprint
from flask import request, Flask,  session, g, redirect, url_for, abort, render_template, flash
from urlparse import urlparse
import subprocess
import os,time,datetime
import sys, urllib as ul
import base64
from decimal import Decimal
import kinterbasdb

from cStringIO import StringIO

from libcpmi.eti9 import draw_eti9,mm
from reportlab.pdfgen import canvas

app = Flask(__name__)

config = 'PROD'

#Connection parameters to SERMO Group database
if config == 'TEST':
    dbhostname = "ldap-master"
    dbpath = "/usr/interbase/base.fdb"
    ldapurl = "ldap://domain-tdp"
    ldaproot = "dc=sermo,dc=fr"
    EXPEDITION_DIR='s:/cegidpmi/expeditions/'
    AVIEXP_DIR = 's:/cegidpmi/aviexp/'
    EXPES_DIR='s:/cegidpmi/expes/%s.txt'
elif config == 'TIS':
    dbhostname = "192.168.149.160"
    dbpath = "/usr/interbase/base.fdb"
    ldapurl = "ldap://srvads"
    ldaproot = "dc=tranquilit,dc=local"
    EXPEDITION_DIR='d:/cegidpmi/expeditions/'
    AVIEXP_DIR = 'd:/cegidpmi/aviexp/'
    EXPES_DIR='d:/cegidpmi/expes/%s.txt'
elif config == 'PROD':
    dbpath = "C:/Program Files/Firebird/bin/BASE.FDB"
    dbhostname = "sqltdp1"
    ldapurl = "ldap://domain-tdp"
    ldaproot = "dc=sermo,dc=fr"
    EXPEDITION_DIR='/home/exchange/cegidpmi/expeditions/'
    AVIEXP_DIR = '/home/exchange/cegidpmi/aviexp/'
    EXPES_DIR='/home/exchange/cegidpmi/expes/%s.txt'


dbusername = "SYSDBA"
dbpassword = "masterkey"




if not os.path.isdir(EXPEDITION_DIR):
    os.makedirs(EXPEDITION_DIR)
if not os.path.isdir(AVIEXP_DIR):
    os.makedirs(AVIEXP_DIR)


#Connection parameters to SERMO LDAP database
# Creates a DB connection to database
def getdb_con():
    con = kinterbasdb.connect(
        dsn=dbhostname+':'+dbpath,
        user=dbusername, password=dbpassword)
    return con

# Execute a query against a db connection. if no commnection is given, takes the global
def select(con,sql,params=[]):
    """
    Execute the sql query and returns a cursor
    """
    # Get a Cursor object that operates in the context of Connection con:
    if params:
        params=[str(p) if type(p) is unicode else p for p in params]
    cur = con.cursor()
    # Execute the SELECT statement:
    cur.execute(sql,params)
    return cur

def datecegid_to_datetime(yyyymmdd):
    """Conversion date cegid en datetime python"""
    if not yyyymmdd.strip():
        return None
    else:
        return datetime.datetime(int(yyyymmdd[0:4]),int(yyyymmdd[4:6]),int(yyyymmdd[6:8]))

def datetimeedi_to_datetime(yyyymmddhhmmss):
    """Conversion date/heure edi en datetime python"""
    if not yyyymmddhhmmss.strip():
        return None
    else:
        return datetime.datetime(int(yyyymmddhhmmss[0:4]),int(yyyymmddhhmmss[4:6]),int(yyyymmddhhmmss[6:8]),int(yyyymmddhhmmss[8:10]),int(yyyymmddhhmmss[10:12]),int(yyyymmddhhmmss[12:14]))

def datetime_to_cegiddate(adatetime):
    """Conversion datetime python en date cegid"""
    return adatetime.strftime('%Y%m%d')

def datetime_to_cegidweek(adatetime):
    """Calcule la annee/semaine en fonction de datetime python"""
    (y,w,dw)=adatetime.isocalendar()
    if adatetime.year<y:
        y=adatetime.year
        w=53
    return "%04i%02i" % (y,w)

def datecegid_to_cegidweek(yyyymmdd):
    """Calcule annee/semaine en fonction de date Cegid"""
    if not yyyymmdd.strip():
        return '      '
    return datetime_to_cegidweek(datecegid_to_datetime(yyyymmdd))


def cegiddate_add(yyyymmdd,days):
    """renvoie une datetime en ajoutant le nombre de jours à la date cegid yyyymmddd"""
    return datecegid_to_datetime(yyyymmdd) + datetime.timedelta(days)


def makecsv(cur):
    """ Create a csv string from a cursor without headers"""
    def colformat(c):
        if c is None:
            return ""
        else:
            return str(c).strip()

    csv=""
    return "\r\n".join( [ ";".join( [colformat(c) for c in r]) for r in cur.fetchall()])


@app.route('/')
def index():
    return render_template('index.html')

@app.route('/colisage/uploadcolisage', methods=['GET','POST'])
def uploadcolisage():
    """Crée un fichier CSV pour la fonction upload colisage aviexp de eCar Tenor
    Si codecde est spécifié, alors un fichier d'expédition codes barres pour CegidPMI est créé

    Params :
        obligatoires : csvbase64,nobl
        facultatifs : cde,codeclient
    """
    csvbase64 = request.values['csvbase64']
    nobl = request.values['nobl']
    codeclient = request.values.get('codeclient',None)

    print "csvbase64 : "+ csvbase64
    print "nobl : "+ nobl
    print "codeclient : "+ codeclient

    # Add some padding at the end so that it fit the multiple of 8 requirement of base64decode
    pad='=' * (8 - len(csvbase64.replace('\r\n','')) % 8)
    csvbase64=csvbase64+pad
    csv=base64.decodestring(csvbase64)
    incols=['nobl','refart','ccart','um','uc','lot','quantcond']
    inrecs=[]
    for r in csv.splitlines():
        if r:
            newrec={}
            csvrec=r.split(';')
            i=0
            for c in incols:
                newrec[c]=csvrec[i]
                i+=1
            inrecs.append(newrec)

    # Trier par refart/ccart pour pouvoir numeroter les palettes et colis [issue1878]
    inrecs.sort(cmp=lambda x,y: cmp(x['refart']+x['ccart']+x['um']+x['uc'], y['refart']+y['ccart']+y['um']+y['uc']))

    # transformer en fichier intégration CSV eCar AVIEXP
    file=open(os.path.join(AVIEXP_DIR,'ecar-%s.csv' % (nobl)),'wb')
    ligneaviexp="%(nobl)s;%(refart)s;%(ccart)s;%(um)s;%(uc)s;%(lot)s"

    noordre=1
    for c in inrecs:
        c['noordre']=noordre
        file.write(ligneaviexp % c)
        file.write('\r\n')
        noordre+=1

    file.close()

    # si un codeclient est fourni, on crée
    #  - une commande fermée dans CegidPMI
    #  - un fichier de colisage pour l'intégration rapide codes barres CegidPMI
    if codeclient:
        # Création fichier expédition CODES Barre Cegid
        # Trier par UM, UC pour pouvoir numeroter les palettes et colis
        # incols=['nobl','refart','ccart','um','uc','lot','quantcond']
        #inrecs.sort(cmp=lambda x,y: cmp(x['um']+x['uc'], y['um']+y['uc']))
        lignescde = []
        for c in inrecs:
            (codeart,ccart,quantcond,refext) = getart_from_codebarre(c['lot'])
            lignescde.append([codeart,ccart,quantcond])

        articles = create_comcli(codeclient=codeclient,
            lignescde=lignescde)
        # dans articles : dictionnaire {(codeart,ccart) : numerocde,numeroligne,quant}
        cde = articles[articles.keys()[0]][0]
        # transformer en fichier integration rapide code barres CegidPMI
        file=open(os.path.join(EXPEDITION_DIR,'exped-%s.txt' % (cde)),'w')

        print inrecs
        #outcols="%(type)s;%(cde)%(ligne)s;%(depot)s;%(lot)s;%(nocolis)s;%(qtecolis)s;%(pdsnet)s;%(pdsnet)s;%(volume)s;%(nopalette)s;%(codeembal)s;%(libembal)s;%(datecol)s;%(um)s;%(uc)s;%(noordre)i;%(pointdest)s"
        lignecegid="X;%(nocde)s%(noligne)s;;%(lot)s;%(noordre)i;%(quantcond)s;;;;;;;;%(um)s;%(uc)s;%(noordre)i;"

        noordre=1
        for c in inrecs:
            c['noordre']=noordre
            (codeart,ccart,quantcond,refext) = getart_from_codebarre(c['lot'])
            # recup la ligne de commande pour l'article scanné
            (c['nocde'],c['noligne']) = articles[(codeart,ccart)][0:2]
            file.write(lignecegid % c)
            file.write('\r\n')
            noordre+=1
        file.close()

    return "OK"


@app.route('/colisage/getnewidum')
def getnewidum():

    con = getdb_con()
    #cur = select("select 1", con)
    SQL_UM="""select * from proc_genum"""

    content = str(select(con,SQL_UM).fetchall()[0][0])
    rv = app.make_response(content)
    rv.mimetype='text/plain'
    rv.headers['Cache-Control'] = "no-store"
    return rv

def getart_from_codebarre(codebarre):
    """Renvoie (codeart,ccart,quantcond,refext)
                de l'article en fonction code barre (OF / n° lot)
        Prends en compte les OF Process avec produits résultants multiples.
            Dans ce cas, le code barre est composé de OF.ligne.no_composant
        Pour les OF classiques, le code barre est OF.ligne
        """
    SQL_REFART_CDE_RESULTANT="""
        select rfctref as REFERENCE,bectcccomp as CCART,arcnpcecon as QTECOND,arktcodart as CODEART
        from besoin
        left join refext on (bectcodcom=rfktcodart and bectcccomp=rfktcomart and bectcompte=rfktcode)
        left join article on (arktcodart=bectcodcom and bectcccomp=arktcomart and arktsoc=bektsoc)
        where bektsoc='100' and bektnumero='%(cde)s' and bektligne='%(ligne)s' and bektnocomp='%(nocomp)s' and bektindpsf='000' and bectfatncp='32'
    """

    SQL_REFART_CDE="""
        select lcctrefext as REFERENCE,lcctcomart as CCART,arcnpcecon as QTECOND,arktcodart as CODEART
        from lcomcli
        left join article on (arktcodart=lcctcodart and arktcomart=lcctcomart and arktsoc=lcktsoc)
        where lcktsoc='100' and lccttype='P' and lcktnumero='%(cde)s' and lcktligne='%(ligne)s' and lcktpsf='000'
        """
    con = getdb_con()
    try:
        (cde,ligne,nocomp)=codebarre.split('.')
        # Teste avec la requête de produit résultants
        ref_ind=select(con,SQL_REFART_CDE_RESULTANT % {'cde':cde,'ligne':ligne,'nocomp':nocomp}).fetchall()
        if ref_ind:
            (refext,ccart,qtecond,codart)=ref_ind[0]
        # Teste avec la requête OF simple
        else:
            ref_ind=select(con,SQL_REFART_CDE % {'cde':cde,'ligne':ligne}).fetchall()
            if ref_ind:
                (refext,ccart,qtecond,codart)=ref_ind[0]
            else:
                (refext,ccart,qtecond,codart)=('','',0.0,'')
    except Exception,e:
        print "no result: %s" % e.message
        (codart,ccart,qtecond,refext)=('','',0.0,'')

    if codart==None:
        codart=''
    if ccart==None:
        ccart=''
    if qtecond==None:
        qtecond=0.0
    if refext==None:
        refext=''
    return (codart.strip(),ccart.strip(),qtecond,refext.strip())


@app.route('/colisage/getrefartcde')
def getrefartcde():
    (codart,ccart,qtecond,refext) = getart_from_codebarre(request.values['codebarre'])
    content =  "%s;%s;%f" % (refext.strip(),ccart.strip(),qtecond)
    rv = app.make_response(content)
    rv.mimetype='text/plain'
    rv.headers['Cache-Control'] = "no-store"
    return rv


@app.route('/colisage/form_eti9')
def form_eti9():
    """Formulaire HTML pour la génération d'étiquettes ETI9"""
    cde = request.values.get('cde','100127')
    ligne = request.values.get('ligne','010')
    no_debut = request.values.get('no_debut','0')
    no_fin = request.values.get('no_fin','100')

    return render_template('form_eti9.html',cde =cde, ligne=ligne,no_debut=no_debut,no_fin=no_fin)

@app.route('/colisage/eti9', methods=['GET','POST'])
def eti9():
    cde = request.values.get('cde','100127')
    ligne = request.values.get('ligne','010')
    no_debut = request.values.get('no_debut','0')
    no_fin = request.values.get('no_fin','100')
    con = getdb_con()


    SQL_ENS_PROCESS="""
        select * from lcomcli where lcktsoc='100' and lcktnumero='%(cde)s' and lcktligne='%(ligne)s' and lcktpsf='000' and lcctfatn='33'
        """

    SQL_ETI9_PROD_RESULTANT="""
        select distinct bektnumero||'.'||bektligne||'.'||bektnocomp as CDE,
        bectcodcom as CODEART,bectcccomp as CCART,lccjdelfab as DATECREAT,
        rfctref as REFERENCE,arcnpdsuni as PDSBRUT,
        lcctrubaff as NOETI,case when refext.RFCTLIB01 is not null and refext.RFCTLIB01<>'' then rtrim(refext.RFCTLIB01)||' '|| rtrim(refext.RFCTLIB02) ELSE rtrim(arctlib01)||' '|| rtrim(arctlib02) end as LIBART,
        arcnpcecon as QTECOND,arctnofou1 as CODECLIENT,
        clctnom as CLIENT,clctnofour as CODEFOURNISSEUR,arctcodcat as PT_DECHARG
        from besoin
        left join lcomcli on (bektsoc=lcktsoc and bektnumero=lcktnumero and  bektligne=lcktligne and bektindpsf=lcktpsf)
        left join article on (arktcodart=bectcodcom and bectcccomp=arktcomart and arktsoc=bektsoc)
        left join client on (bectcompte=clktcode and clktsoc=lcktsoc)
        left join refext on (bectcodcom=rfktcodart and bectcccomp=rfktcomart and bectcompte=rfktcode)
        where bektsoc='100' and bektnumero='%(cde)s' and bektligne='%(ligne)s' and bektindpsf='000' and bectfatncp='32'
        """

    SQL_ETI9="""
        select lcktnumero||'.'||lcktligne||'.'||lcktpsf as CDE,
        lcctcodart as CODEART,lcctcomart as CCART,lccjdelfab as DATECREAT,
        lcctrefext as REFERENCE,arcnpdsuni as PDSBRUT,
        lcctrubaff as NOETI,
        case when refext.RFCTLIB01 is not null and refext.RFCTLIB01<>'' then rtrim(refext.RFCTLIB01)||' '|| rtrim(refext.RFCTLIB02) ELSE rtrim(arctlib01)||' '|| rtrim(arctlib02) end as LIBART,
        arcnpcecon as QTECOND,arctnofou1 as CODECLIENT,
        clctnom as CLIENT,clctnofour as CODEFOURNISSEUR,arctcodcat as PT_DECHARG
        from lcomcli
        left join article on (arktcodart=lcctcodart and arktcomart=lcctcomart and arktsoc=lcktsoc)
        left join client on (arctnofou1=clktcode and clktsoc=lcktsoc)
        left join refext on (rfktcodart=lcctcodart and rfktcomart=lcctcomart and clktcode=rfktcode)
        where lcktsoc='100' and lccttype='P' and lcktnumero='%(cde)s' and lcktligne='%(ligne)s' and lcktpsf='000'
        """


    """Création d'un PDF d'étiquettes ETI9 pour les pièces d'un OF"""
    # Recup si OF d'un article ensemble process
    ofprocess=select(con,SQL_ENS_PROCESS % {'cde':cde,'ligne':ligne}).fetchonemap()

    if ofprocess:
      # Recuperation de l'enregistrement
      ofdatas=select(con,SQL_ETI9_PROD_RESULTANT % {'cde':cde,'ligne':ligne}).fetchallmap()
    else:
      # Recuperation de l'enregistrement
      ofdatas=select(con,SQL_ETI9 % {'cde':cde,'ligne':ligne}).fetchallmap()

    if ofdatas:
        # Creation d'un fichier en memoire pour stocker le PDF temporaire
        pdf=StringIO()
        c=canvas.Canvas(pdf)

        for ofdata in ofdatas:
          #Suppression des espaces avant apres des champs
          # r est un map des champs.
          r={}
          for col in ofdata.iteritems():
            if isinstance(col[1],str):
                r[col[0]]=col[1].strip().decode('iso8859').encode('utf8')
            else:
                r[col[0]]=col[1]


          # Les numeros d'etiquette debut-fin sont sous la forme "5-20" dans le champs NOETI
          if (no_debut<>'') and (no_fin<>''):
            noeti_debut=int(no_debut)
            noeti_fin=int(no_fin)
          elif r['NOETI']:
            noeti_debut=int(r['NOETI'].split('-')[0])
            noeti_fin=int(r['NOETI'].split('-')[1])
          else:
            noeti_debut=0
            noeti_fin=0

          if noeti_fin-noeti_debut > 1000:
            raise Exception('Nombre etiquettes limite : 1000')
          hauteur_bande=74
          inter_bande=2
          marge=5

          y=297-marge-hauteur_bande
          noeti=noeti_debut
          empty=True

          while noeti<=noeti_fin:
            draw_eti9(c,0,y,
                destinataire=r['CLIENT'][0:28],
                pt_decharg=r['PT_DECHARG'],
                pt_destination='',
                expediteur='SERMO AIGREFEUILLE',
                code_routage='',
                code_produit=r['REFERENCE'] or r['CODEART'],
                quant=r['QTECOND'],
                ref_fournisseur=r['CDE'],
                no_etiq=noeti,
                poids_net=Decimal('1.0')*r['PDSBRUT']*r['QTECOND'],
                poids_brut=Decimal('1.0')*r['PDSBRUT']*r['QTECOND'],
                date=r['DATECREAT'][2:],
                nb=1,
                description=r['LIBART'],
                no_lot=r['CDE'],
                ind_modif=r['CCART'],
                code_vendeur=r['CODEFOURNISSEUR']
            )
            empty=False
            # Si on n'a plus la place de placer une etiquette
            if y-inter_bande-hauteur_bande < marge:
                c.showPage()
                y=297-marge-hauteur_bande
                empty=True
            else:
                y-=hauteur_bande+inter_bande
            noeti+=1

          if not empty:
            c.showPage()
          c.save()
        # renvoie la chaine de caractere PDF

        content = pdf.getvalue()
        print content

        rv = app.make_response(content)
        rv.mimetype='application/pdf'
        rv.headers['Content-Type']="application/pdf"
        rv.headers['Content-Disposition']="attachment; filename=%s.pdf" % ("%s-%s-%s" %("eti9",cde,ligne))
        rv.headers['Cache-Control'] = "no-store"
        return rv
    else:
        return render_template('form_eti9.html',cde =cde, ligne=ligne,no_debut=no_debut,no_fin=no_fin)

@app.route('/colisage/ral')
def ral():
    """Reste à livrer sous forme CSV
    cde,codeclient, Client, codeart, ccart,  QuanCond, Lib, QteCde, QteExp, delaiexp, QTERAL,ColisRAL,NBUCUM
        Attention : le champ arcnpcecon des articles doit être <>0
    """

    SQL_RAL="""
        SELECT lcktnumero||'.'||lcktligne as cde,lcctcode as codeclient, clctnom as Client, lcctcodart as codeart, lcctcomart as ccart,  arcnpcecon as QuanCond,
        rtrim(lcctlib01)||' '||rtrim(lcctlib02) as Lib,lccnqtecde as QteCde, lccnqteexp as QteExp, lccjdelexp as delaiexp,
        (lccnqtecde-lccnqteexp) as QTERAL,(lccnqtecde-lccnqteexp)/case arcnpcecon when 0 then 1 else arcnpcecon end as ColisRAL,rtrim(article.arctartrep) as NBUCUM
        FROM LCOMCLI
        LEFT JOIN CLIENT on lcktsoc=clktsoc and lcctcode=clktcode
        LEFT JOIN ARTICLE on lcktsoc=arktsoc and lcctcodart=arktcodart and lcctcomart=arktcomart
        WHERE
        LCKTSOC='100' AND LCCTTYPE='S' AND
        SUBSTR(LCCTSOLACE,1,1)<>'S' AND
        LCCTCODE<>'009999' AND
        (lccnqtecde-lccnqteexp)>0.001 AND
        lccjdelexp>=? AND
        lccjdelexp<=?
        order by lccjdelexp
        """

    con = getdb_con()
    rv = app.make_response(makecsv(select(con,SQL_RAL,(
        datetime_to_cegiddate(datetime.datetime.today()-datetime.timedelta(days=180)) ,
        datetime_to_cegiddate(datetime.datetime.today()+datetime.timedelta(days=60)) ,
        ))))
    rv.mimetype='text/csv'
    rv.headers['Content-Type'] = "text/csv"
    rv.headers["Content-Disposition"] = "attachment; filename=ral-%s.csv" % datetime.datetime.today().strftime('%Y%m%d')
    rv.headers["Cache-Control"] = "no-store"
    return rv



@app.route('/colisage/ofart')
def ofart():
    """Info traçabilité sur les OFs produits pour un Article sous forme CSV
        NOOF,codeart, ccart, Lib, delai
    """
    SQL_OFART="""
        SELECT lcktnumero||'.'||lcktligne ||'.'||lcktpsf as NOOF,lcctcodart as codeart, lcctcomart as ccart, lcctlib01||' '||lcctlib02 as Lib, lccjdelexp as delai
        FROM LCOMCLI
        WHERE
        LCKTSOC='100' AND LCCTTYPE='P'
        and lcctcodart='%(codeart)s' and lcctcomart='%(ccart)s'
        """

    con = getdb_con()
    codeart = request.values['codeart']
    ccart = request.values['ccart']
    print SQL_OFART % {'codeart':codeart,'ccart':ccart}
    con = getdb_con()
    rv = app.make_response(makecsv(select(con,SQL_OFART % {'codeart':codeart,'ccart':ccart})))
    rv.mimetype="text/csv"
    rv.headers["Content-Type"] = "text/csv"
    rv.headers["Content-Disposition"] = "attachment; filename=ofart-%s.csv" % datetime.datetime.today().strftime('%Y%m%d')
    rv.headers["Cache-Control"] = "no-store"
    return rv

@app.route('/colisage/liste_clients')
def liste_clients():
    """Liste des clients récents
    """
    SQL_CLIENTS = """
    select distinct l.LCCTCODE||' '||rtrim(c.CLCTABRE) as lib, c.clctcodfac from LCOMOUV l
        left join client c on c.CLKTCODE=l.LCCTCODE and c.clktSOC=l.lcktsoc
        where l.LCCJDELEXP between ? and ?
    union
    select distinct l.LCCTCODE||' '||rtrim(c.CLCTABRE) as lib, c.clctcodfac from LCOMCLI l
        left join client c on c.CLKTCODE=l.LCCTCODE and c.clktSOC=l.lcktsoc
        where l.LCCJDELEXP between ? and ?
    """
    con = getdb_con()
    content = '\n'.join([r[0] for r in
        select(con,SQL_CLIENTS,params=[\
            datetime_to_cegiddate(datetime.datetime.today() - datetime.timedelta(days=60)),
            datetime_to_cegiddate(datetime.datetime.today() + datetime.timedelta(days=60)),
            datetime_to_cegiddate(datetime.datetime.today() - datetime.timedelta(days=60)),
            datetime_to_cegiddate(datetime.datetime.today() + datetime.timedelta(days=60)),
        ]).fetchall()])

    rv = app.make_response(content)
    rv.mimetype="text/ascii"
    rv.headers["Content-Type"] = "text/ascii"
    #rv.headers["Content-Disposition"] = "attachment; filename=clients.txt"
    rv.headers["Cache-Control"] = "no-store"
    return rv


@app.route('/colisage/check_codeclient')
def check_codeclient():
    """ Verifie si le code client PARAM codeclient existe
       renvoie le libellé et la ville
    """
    SQL_CHECKCLIENT = """
        select rtrim(c.CLCTNOM)||' ('||rtrim(c.CLCTVILLE)||')' as lib from client c where c.CLKTSOC='100' and c.CLKTCODE=?
    """
    con = getdb_con()
    # query params must be string, not unicode
    codeclient = str(request.values['codeclient'])
    row = select(con,SQL_CHECKCLIENT,params=[codeclient,]).fetchone()
    if row:
        content = row[0]
    else:
        content = ''
    rv = app.make_response(content)
    rv.mimetype='text/ascii'
    rv.headers["Content-Type"] = "text/plain"
    rv.headers["Cache-Control"] = "no-store"
    return rv

def getnewnumerocde():
    """Reserve un numero de cde ferme"""
    con = getdb_con()
    r = select(con,"SELECT PACNUM01 from PARAM WHERE PAKTSOC = '100' AND PAKTETS = ' ' AND PAKTCODE = '1' AND PAKTNOPAR = '056'").fetchall()
    select(con,"UPDATE PARAM SET PACNUM01=? WHERE PAKTSOC = '100' AND PAKTETS = ' ' AND PAKTCODE = '1' AND PAKTNOPAR = '056'",(r[0][0]+1,))
    numero = "%06.0f" % (r[0][0]+1,)
    con.commit()
    return numero

def create_comcli(numerocde='',codeclient='SERMO',delai=None,lignescde=[]):
    """Cree une commande ferme pour le client codeclient, avec le numero de cde fourni
        les lignes de commande est un tableau  ;
            [codeart,ccart,quant]
        retourne un dictionnaire qui donne la ligne de commande en fonction du code article:
           {(codeart,ccart): ('cde','ligne'),}
    """

    SQL_ECOMCLI_INSERT="""INSERT INTO ECOMCLI (ECKTSOC, ECKTNUMERO, ECKTINDICE, ECCTETS, ECCTTYPE1,
        ECCTTYPE2, ECCJCRE, ECCTCODE, ECCTNOM, ECCTRUE1, ECCTRUE2, ECCTCP,
        ECCTVILLE, ECCTPAYS, ECCTCPAYS, ECCTNOMLIV, ECCTRUE1LI, ECCTRUE2LI,
        ECCTCPLIV, ECCTVILLIV, ECCTPAYSLI, ECCTCPAYLI, ECCTREFCDE, ECCTPORT,
        ECCTEMBA, ECCTTRANPO, ECCTREGFAC, ECCTREGBL, ECCTCECHEA, ECCTBECHEA,
        ECCTCREG, ECCTMREG, ECCJECHFOR, ECCTRUPEDI, ECCTREGFIS, ECCTDERBL,
        ECCTDERFA, ECCJDERBL, ECCJDERFA, ECCTACTIVI, ECCTBLOCAG, ECCTFRFIXE,
        ECCNPDSBRU, ECCNPDSNET, ECCNVOLUME, ECCNBCOLIS, ECCNBPAL, ECCNESCAGI,
        ECCTRUPBL, ECCTRUPFAC, ECCTDEVISE, ECCNTXDEVI, ECCTREVDOC, ECCTENVOIF,
        ECCTTOP01, ECCTTOP02, ECCTTOP03, ECCTTOP04, ECCTTOP05, ECCTCODFAC,
        ECCTSTATUT, ECCTVISEUR, ECCTRESPON, ECCTPRESCR, ECCTTITRE, ECCTCENACH,
        ECCTSUIREF, ECCTUTIMOD, ECCTBP01, ECCNLIBRE1, ECCNLIBRE2, ECCTTOP06,
        ECCTTOP07, ECCTTOP08, ECCTTOP09, ECCTLINCO)
        VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)
        """

    SQL_LCOMCLI_INSERT="""INSERT INTO LCOMCLI (LCKTSOC, LCKTNUMERO, LCKTLIGNE, LCKTPSF, LCCTETS, LCCTTYPE,
        LCCJCRE, LCCTCODE, LCCTTYPART, LCCTNATURE, LCCTCODART, LCCTCOMART,
        LCCTLIB01, LCCNQTECDE, LCCNPDSCDE, LCCNVOLCDE, LCCNLONCDE, LCCNLARCDE,
        LCCNEPACDE, LCCTUNICDE, LCCTUNISTO, LCCTUNIFAC, LCCNPUBRUT, LCCNRHECR,
        LCCNPUNET, LCCTEXPU, LCCNRHLIG, LCCNRHBAS1, LCCNRHBAS2, LCCNRHBAS3,
        LCCNQTEEXP, LCCNPDSEXP, LCCNVOLEXP, LCCTSOLACE, LCCJDEREX, LCCTNOEXP,
        LCCNQTECON, LCCNPDSCON, LCCNVOLCON, LCCTSOLACC, LCCJDERCO, LCCTCREP1,
        LCCTCREP2, LCCNTXREP1, LCCNTXREP2, LCCNBASCOM, LCCSDELEXP, LCCJDELEXP,
        LCCTNUMLOT, LCCTDEPOT, LCCTNOAFFA, LCCTNOLAFF, LCCSDELDEM, LCCSDELFAB,
        LCCJDELFAB, LCCNPUREVI, LCCTSECART, LCCTSECCDE, LCCTTOPRAL, LCCTTOPBES,
        LCCTTOPEDI, LCCTETACDE, LCCTREVPRI, LCCTPRIORI, LCCTEDIPU, LCCTCODTVA,
        LCCTCODTAR, LCCTDEVISE, LCCNTXDEVI, LCCTREFCDE, LCCTCONDIT, LCCTIMPUT,
        LCCTGAMREM, LCCTAFFMAC, LCCTENVOIF, LCCTZONLIV, LCCTCODEMP, LCCTREPERE,
        LCCTTYPLIB, LCCNVALCDE, LCCTLANCEM, LCCTLIGLAN, LCCTFATN, LCCNBASMAR,
        LCCTLIB02, LCCTSOLACQ, LCCNDENSIT, LCCJDELDDE, LCCJDELARR, LCCSDELARR,
        LCCTANNOTA, LCCTSTATUT, LCCTREFEXT, LCCTPALLOC, LCCTDATREF, LCCTCERCON,
        LCCJCERCON, LCCT12DELP, LCCT12DELD, LCCNQTELAN, LCCNCUMQRE, LCCTTOPGE1,
        LCCTTOPGE2, LCCTFAMART, LCCTRUBAFF, LCCTAFFACH, LCCTLIGACH, LCCNLIBRE1,
        LCCNLIBRE2, LCCTTOP01, LCCTTOP02, LCCTTOP03, LCCTTOP04)
        VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,
        ?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?
        );
        """

    if not delai:
        delai = datetime_to_cegiddate(datetime.datetime.today())
    if not numerocde:
        numerocde = getnewnumerocde()
    con = getdb_con()
    try:
        # Paramètres client
        PARAM_CLIENT = """SELECT
                CLKTSOC,CLKTCODE,
                CLCTNOM,CLCTRUE1,CLCTRUE2,CLCTBP,CLCTCP,CLCTVILLE,CLCTPAYS,CLCTCPAYS,
                CLCTNOMLIV, CLCTRUE1LI, CLCTRUE2LI, CLCTCPLIV, CLCTVILLIV, CLCTPAYSLI, CLCTCPAYLI,
                CLCTCECHEA,CLCTBECHEA,CLCTCECHEA,CLCTBECHEA,CLCTREGFAC, CLCTCREG, CLCTMREG, CLCTREGFIS, CLCTREGBL, CLCTCODFAC
                FROM CLIENT WHERE CLKTSOC = '100' AND CLKTCODE = ?"""
        (CLKTSOC,CLKTCODE,
                CLCTNOM,CLCTRUE1,CLCTRUE2,CLCTBP,CLCTCP,CLCTVILLE,CLCTPAYS,CLCTCPAYS,
                CLCTNOMLIV, CLCTRUE1LI, CLCTRUE2LI, CLCTCPLIV, CLCTVILLIV, CLCTPAYSLI, CLCTCPAYLI,
                CLCTCECHEA,CLCTBECHEA,CLCTCECHEA,CLCTBECHEA,CLCTREGFAC, CLCTCREG, CLCTMREG, CLCTREGFIS, CLCTREGBL,CLCTCODFAC)=\
                select(con,PARAM_CLIENT,(codeclient,)).fetchone()

        # Insertion entête commande
        ECKTSOC = '100'
        ECKTNUMERO = numerocde
        ECKTINDICE = '000'
        ECCTTYPE1 = 'C'
        ECCTTYPE2 = 'S'
        ECCJCRE = datetime_to_cegiddate(datetime.datetime.today())
        ECCTCODE = CLKTCODE
        ECCTNOM = CLCTNOM
        ECCTRUE1 = CLCTRUE1
        ECCTRUE2 = CLCTRUE2
        ECCTCP = CLCTCP
        ECCTVILLE = CLCTVILLE
        ECCTPAYS = CLCTPAYS
        ECCTCPAYS = CLCTCPAYS
        ECCTNOMLIV = CLCTNOMLIV
        ECCTRUE1LI = CLCTRUE1LI
        ECCTRUE2LI = CLCTRUE2LI
        ECCTCPLIV = CLCTCPLIV
        ECCTVILLIV = CLCTVILLIV
        ECCTPAYSLI = CLCTPAYSLI
        ECCTCPAYLI = CLCTCPAYLI
        ECCTREGFAC = CLCTREGFAC
        ECCTREGBL = CLCTREGBL
        ECCTCECHEA = CLCTCECHEA
        ECCTBECHEA = CLCTBECHEA
        ECCTCREG = CLCTCREG
        ECCTMREG = CLCTMREG
        ECCTRUPEDI = '17'
        ECCTREGFIS = '-1'
        ECCTFRFIXE = '2'
        ECCNPDSBRU = 0.0
        ECCNPDSNET = 0.0
        ECCNVOLUME = 0.0
        ECCNBCOLIS = 0.0
        ECCNBPAL = 0.0
        ECCNESCAGI = 0.0
        ECCTDEVISE = 'EUR'
        ECCNTXDEVI = 1.0
        ECCTUTIMOD = 'Colisage'
        ECCTETS=''
        ECCTREFCDE=''
        ECCTPORT=''
        ECCTEMBA=''
        ECCTTRANPO=''
        ECCJECHFOR=''
        ECCTDERBL=''
        ECCJDERBL=''
        ECCJDERFA=''
        ECCTDERFA=''
        ECCTACTIVI=''
        ECCTBLOCAG=''
        ECCTREVDOC=''
        ECCTENVOIF=''
        ECCTTOP01=''
        ECCTTOP02=''
        ECCTTOP03=''
        ECCTTOP04=''
        ECCTTOP05=''
        ECCTCODFAC=CLCTCODFAC
        ECCTSTATUT=''
        ECCTVISEUR=''
        ECCTRESPON=''
        ECCTPRESCR=''
        ECCTTITRE=''
        ECCTCENACH=''
        ECCTSUIREF=''
        ECCTBP01=''
        ECCNLIBRE1=0.0
        ECCNLIBRE2=0.0
        ECCTTOP06=''
        ECCTTOP07=''
        ECCTTOP08=''
        ECCTTOP09=''
        ECCTLINCO=''
        ECCTRUPBL=''
        ECCTRUPFAC=''

        select(con,SQL_ECOMCLI_INSERT,(ECKTSOC, ECKTNUMERO, ECKTINDICE, ECCTETS, ECCTTYPE1,
            ECCTTYPE2, ECCJCRE, ECCTCODE, ECCTNOM, ECCTRUE1, ECCTRUE2, ECCTCP,
            ECCTVILLE, ECCTPAYS, ECCTCPAYS, ECCTNOMLIV, ECCTRUE1LI, ECCTRUE2LI,
            ECCTCPLIV, ECCTVILLIV, ECCTPAYSLI, ECCTCPAYLI, ECCTREFCDE, ECCTPORT,
            ECCTEMBA, ECCTTRANPO, ECCTREGFAC, ECCTREGBL, ECCTCECHEA, ECCTBECHEA,
            ECCTCREG, ECCTMREG, ECCJECHFOR, ECCTRUPEDI, ECCTREGFIS, ECCTDERBL,
            ECCTDERFA, ECCJDERBL, ECCJDERFA, ECCTACTIVI, ECCTBLOCAG, ECCTFRFIXE,
            ECCNPDSBRU, ECCNPDSNET, ECCNVOLUME, ECCNBCOLIS, ECCNBPAL, ECCNESCAGI,
            ECCTRUPBL, ECCTRUPFAC, ECCTDEVISE, ECCNTXDEVI, ECCTREVDOC, ECCTENVOIF,
            ECCTTOP01, ECCTTOP02, ECCTTOP03, ECCTTOP04, ECCTTOP05, ECCTCODFAC,
            ECCTSTATUT, ECCTVISEUR, ECCTRESPON, ECCTPRESCR, ECCTTITRE, ECCTCENACH,
            ECCTSUIREF, ECCTUTIMOD, ECCTBP01, ECCNLIBRE1, ECCNLIBRE2, ECCTTOP06,
            ECCTTOP07, ECCTTOP08, ECCTTOP09, ECCTLINCO) )
        result = {}
        numeroligne=0

        # regroupement lignes de commande par code/cc
        class artquant:
            def __init__(self,numerocde='000000',numeroligne='000',quant=0.0):
                self.numerocde=numerocde;self.numeroligne=numeroligne;self.quant=quant

        articles = {}
        for (codeart,ccart,quant) in lignescde:
            if not (codeart,ccart) in articles:
                numeroligne += 10
                # initialisation quant et ligne
                articles[(codeart,ccart)] = [numerocde,"%03.0f" % numeroligne,quant]
            else:
                a = articles[(codeart,ccart)]
                # quantité
                a[2] += quant

        # Creation lignes de commande
        for (codeart,ccart) in articles:
            (ARCTLIB01,ARCNPCECON,ARCNPDSUNI,ARCTEXPU,ARCNTARIF9,ARCTCODFAM ) =\
                select(con,"select ARCTLIB01,ARCNPCECON,ARCNPDSUNI,ARCTEXPU,ARCNTARIF9,ARCTCODFAM from ARTICLE where ARKTSOC='100' and arktcodart=? and arktcomart=?",(codeart,ccart)).fetchone()
            if not ARCNPCECON:
                ARCNPCECON = 1.0

            LCKTSOC='100'
            LCKTNUMERO=numerocde
            LCKTLIGNE=articles[(codeart,ccart)][1]
            LCKTPSF='000'
            LCCTETS=''
            LCCTTYPE='S'
            LCCJCRE=datetime_to_cegiddate(datetime.datetime.today())
            LCCTCODE=codeclient
            LCCTTYPART='SF'
            LCCTNATURE='6'
            LCCTCODART=codeart
            LCCTCOMART=ccart
            LCCTLIB01=ARCTLIB01
            LCCNQTECDE=Decimal(str(articles[(codeart,ccart)][2]))
            LCCNPDSCDE='0.0'
            LCCNVOLCDE='0.0'
            LCCNLONCDE='0.000'
            LCCNLARCDE='0.000'
            LCCNEPACDE='0.000'
            LCCTUNICDE='u'
            LCCTUNISTO='u'
            LCCTUNIFAC=''
            LCCNPUBRUT=ARCNTARIF9
            LCCNRHECR='0.00'
            LCCNPUNET=ARCNTARIF9
            LCCTEXPU=ARCTEXPU
            LCCNRHLIG='0.00'
            LCCNRHBAS1='0.00'
            LCCNRHBAS2='0.00'
            LCCNRHBAS3='0.00'
            LCCNQTEEXP='0.000'
            LCCNPDSEXP='0.000'
            LCCNVOLEXP='0.000'
            LCCTSOLACE=''
            LCCJDEREX=''
            LCCTNOEXP=''
            LCCNQTECON='0.000'
            LCCNPDSCON='0.000'
            LCCNVOLCON='0.000'
            LCCTSOLACC=''
            LCCJDERCO=''
            LCCTCREP1=''
            LCCTCREP2=''
            LCCNTXREP1='0.00'
            LCCNTXREP2='0.00'
            LCCNBASCOM='0.00'
            LCCSDELEXP=datetime_to_cegidweek(datetime.datetime.today())
            LCCJDELEXP=datetime_to_cegiddate(datetime.datetime.today())
            LCCTNUMLOT=''
            LCCTDEPOT=''
            LCCTNOAFFA=LCKTNUMERO
            LCCTNOLAFF='001'
            LCCSDELDEM='      '
            LCCSDELFAB='      '
            LCCJDELFAB='        '
            LCCNPUREVI='0.00'
            LCCTSECART=''
            LCCTSECCDE=''
            LCCTTOPRAL=''
            LCCTTOPBES=''
            LCCTTOPEDI=''
            LCCTETACDE=''
            LCCTREVPRI=''
            LCCTPRIORI=''
            LCCTEDIPU=''
            LCCTCODTVA='1'
            LCCTCODTAR='9'
            LCCTDEVISE='EUR'
            LCCNTXDEVI='1.000000'
            LCCTREFCDE=''
            LCCTCONDIT=''
            LCCTIMPUT=''
            LCCTGAMREM=''
            LCCTAFFMAC=ARCTCODFAM
            LCCTENVOIF=''
            LCCTZONLIV=''
            LCCTCODEMP=''
            LCCTREPERE=''
            LCCTTYPLIB='O'
            if LCCTEXPU=='M':
                LCCNVALCDE = LCCNQTECDE * LCCNPUNET / Decimal('1000.0')
            elif LCCTEXPU=='C':
                LCCNVALCDE = LCCNQTECDE * LCCNPUNET / Decimal('100.0')
            else:
                LCCNVALCDE = LCCNQTECDE * LCCNPUNET
            LCCTLANCEM='      '
            LCCTLIGLAN='   '
            LCCTFATN='01'
            LCCNBASMAR='0.000'
            LCCTLIB02='                              '
            LCCTSOLACQ=' '
            LCCNDENSIT='0.000'
            LCCJDELDDE='        '
            LCCJDELARR='        '
            LCCSDELARR='      '
            LCCTANNOTA='          '
            LCCTSTATUT='  '
            LCCTREFEXT='                        '
            LCCTPALLOC='   '
            LCCTDATREF='               '
            LCCTCERCON='      '
            LCCJCERCON='        '
            LCCT12DELP=' '
            LCCT12DELD=' '
            LCCNQTELAN='0.000'
            LCCNCUMQRE='0.000'
            LCCTTOPGE1=' '
            LCCTTOPGE2=' '
            LCCTFAMART='    '
            LCCTRUBAFF='          '
            LCCTAFFACH='      '
            LCCTLIGACH='   '
            LCCNLIBRE1='0.000'
            LCCNLIBRE2='0.000'
            LCCTTOP01=' '
            LCCTTOP02=' '
            LCCTTOP03=' '
            LCCTTOP04=' '

            select(con,SQL_LCOMCLI_INSERT,(LCKTSOC, LCKTNUMERO, LCKTLIGNE, LCKTPSF, LCCTETS, LCCTTYPE,
                LCCJCRE, LCCTCODE, LCCTTYPART, LCCTNATURE, LCCTCODART, LCCTCOMART,
                LCCTLIB01, LCCNQTECDE, LCCNPDSCDE, LCCNVOLCDE, LCCNLONCDE, LCCNLARCDE,
                LCCNEPACDE, LCCTUNICDE, LCCTUNISTO, LCCTUNIFAC, LCCNPUBRUT, LCCNRHECR,
                LCCNPUNET, LCCTEXPU, LCCNRHLIG, LCCNRHBAS1, LCCNRHBAS2, LCCNRHBAS3,
                LCCNQTEEXP, LCCNPDSEXP, LCCNVOLEXP, LCCTSOLACE, LCCJDEREX, LCCTNOEXP,
                LCCNQTECON, LCCNPDSCON, LCCNVOLCON, LCCTSOLACC, LCCJDERCO, LCCTCREP1,
                LCCTCREP2, LCCNTXREP1, LCCNTXREP2, LCCNBASCOM, LCCSDELEXP, LCCJDELEXP,
                LCCTNUMLOT, LCCTDEPOT, LCCTNOAFFA, LCCTNOLAFF, LCCSDELDEM, LCCSDELFAB,
                LCCJDELFAB, LCCNPUREVI, LCCTSECART, LCCTSECCDE, LCCTTOPRAL, LCCTTOPBES,
                LCCTTOPEDI, LCCTETACDE, LCCTREVPRI, LCCTPRIORI, LCCTEDIPU, LCCTCODTVA,
                LCCTCODTAR, LCCTDEVISE, LCCNTXDEVI, LCCTREFCDE, LCCTCONDIT, LCCTIMPUT,
                LCCTGAMREM, LCCTAFFMAC, LCCTENVOIF, LCCTZONLIV, LCCTCODEMP, LCCTREPERE,
                LCCTTYPLIB, LCCNVALCDE, LCCTLANCEM, LCCTLIGLAN, LCCTFATN, LCCNBASMAR,
                LCCTLIB02, LCCTSOLACQ, LCCNDENSIT, LCCJDELDDE, LCCJDELARR, LCCSDELARR,
                LCCTANNOTA, LCCTSTATUT, LCCTREFEXT, LCCTPALLOC, LCCTDATREF, LCCTCERCON,
                LCCJCERCON, LCCT12DELP, LCCT12DELD, LCCNQTELAN, LCCNCUMQRE, LCCTTOPGE1,
                LCCTTOPGE2, LCCTFAMART, LCCTRUBAFF, LCCTAFFACH, LCCTLIGACH, LCCNLIBRE1,
                LCCNLIBRE2, LCCTTOP01, LCCTTOP02, LCCTTOP03, LCCTTOP04))

        # Mise à jour cumuls stocks
        for (codeart,ccart) in articles:
            select(con,'execute procedure PROC_Q1CUMQTE_ARTICLE(?,?,?)',(codeart,ccart,datetime_to_cegiddate(datetime.datetime.today())))

        con.commit()
        return articles
    except:
        con.rollback()
        raise

@app.route('/colisage/update_colisage', methods=['GET','POST'])
def update_colisage():
    """Crée un fichier CSV pour la fonction expédition rapide code barre de CegidPMI"""
    csvbase64 = request.values['csvbase64']
    cde = request.values['cde']

    # Add some padding at the end so that it fit the multiple of 8 requirement of base64decode
    pad='=' * (8 - len(csvbase64.replace('\r\n','')) % 8)
    csvbase64=csvbase64+pad
    csv=base64.decodestring(csvbase64)
    incols=['cde','codeart','ccart','lot','uc','um','quantcond']
    inrecs=[]
    for r in csv.splitlines():
        if r:
            newrec={}
            csvrec=r.split(';')
            i=0
            for c in incols:
                newrec[c]=csvrec[i]
                i+=1
            inrecs.append(newrec)
    # Trier par UM, UC pour pouvoir numeroter les palettes et colis
    inrecs.sort(cmp=lambda x,y: cmp(x['um']+x['uc'], y['um']+y['uc']))
    # Verif meme cde

    # Verifier quant/article par rapport a la commande
    # Si incomplet : BL sera acompte sinon solde.

    # transformer en fichier integration CegidPMI
    file=open(os.path.join(EXPEDITION_DIR,'exped-%s.txt' % (cde)),'w')
    print inrecs
    #outcols="%(type)s;%(cde)%(ligne)s;%(depot)s;%(lot)s;%(nocolis)s;%(qtecolis)s;%(pdsnet)s;%(pdsnet)s;%(volume)s;%(nopalette)s;%(codeembal)s;%(libembal)s;%(datecol)s;%(um)s;%(uc)s;%(noordre)i;%(pointdest)s"
    lignecegid="X;%(nocde)s%(noligne)s;;%(lot)s;%(noordre)i;%(quantcond)s;;;;;;;;%(um)s;%(uc)s;%(noordre)i;"

    noordre=1
    for c in inrecs:
        c['noordre']=noordre
        (c['nocde'],c['noligne'])=c['cde'].split('.')
        file.write(lignecegid % c)
        file.write('\r\n')
        noordre+=1
    file.close()
    return "OK"



if __name__ == '__main__':
    #create_comcli(codeclient='POAREV',lignescde=[
    #    ('PFP95GT20105717','A',100.0),
    #    ('PFPBDK9570102142','',200.0),
    #    ('PFP95GT20105717','A',150.0),
    #    ])
    app.run(host="0.0.0.0",port=8202,debug=True)
