#!/usr/bin/python2.4
# -*- coding: UTF-8 -*-

#-----------------------------------------------------------------------------
# Name:        cegidpmiapp.py
# Purpose:     CherryPy SERMO with Login / access to db Application template
#
# Author:      Hubert TOUVET
#
# Created:     2006/08/28
# RCS-ID:      $Id: cegidpmiapp.py $
# Copyright:   (c) 2004
# Licence:     GPL
#-----------------------------------------------------------------------------


import sys
import kinterbasdb
import ldap,ldap.ldapobject
#import cherrypy
#from cherrypy.lib import httptools

TEST=True

#Connection parameters to SERMO Group database
if TEST:
    dbhostname = "192.168.149.160"
    dbpath = "/usr/interbase/base.fdb"
    ldapurl = "ldap://srvads"
    ldaproot = "dc=tranquilit,dc=local"

else:
    dbpath = "C:/Program Files/Firebird/bin/BASE.FDB"
    dbhostname = ""
    ldapurl = "ldap://domain-tdp"
    ldaproot = "dc=sermo,dc=fr"



dbusername = "SYSDBA"
dbpassword = "masterkey"

#Connection parameters to SERMO LDAP database
# Creates a DB connection to database
def getdb_con():
    con = kinterbasdb.connect(
        dsn=dbhostname+':'+dbpath,
        user=dbusername, password=dbpassword)
    return con

# Execute a query against a db connection. if no commnection is given, takes the global
def select(sql,con):
    """
    Execute the sql query and returns a cursor
    """
    try:
        # Get a Cursor object that operates in the context of Connection con:
        cur = con.cursor()

        # Execute the SELECT statement:
        cur.execute(sql)
    except:
        if con:
            con.close()
        con=None

    return cur

class CegidpmiApp:
    """
    Generic Cherrypy applet template for SERMO
    - CegidPMI database connection
    - Table template
    """
    def __init__(self):
        self._dbcon=None
        self._ldapcon=None

    def db_con(self):
        if not self._dbcon:
            self._dbcon=getdb_con()
        return self._dbcon

    def select(self,sql):
        try:
            return select(sql,self.db_con())
        except kinterbasdb.OperationalError:
            #in case of a failure (database restart...)
            #retry the query after resetting the database connection
            self._dbcon=None
            return select(sql,self.db_con())

    def table(self,cur,callback=None):
        """
            cur est un cursor issu d'une requete
            callback est une fonction qui prend (rowmap,fieldname,value)
            et renvoie une representation texte
        """
        head="<tr>"+"".join(["<th>"+c[0]+"</th>" for c in cur.description])+"</tr>"
        lines=""
        if callback:
            for r in cur.itermap():
                lines=lines+"<tr>"+"".join(["<td>"+callback(r,c[0],c[1])+"</td>" for c in r.iteritems()])+"</tr>"
        else:
            for r in cur:
                lines=lines+"<tr>"+"".join(["<td>"+str(c)+"</td>" for c in r])+"</tr>"

        return "<table border=1  cellpadding=2 cellspacing=0>%s%s</table>" % (head,lines)

    def ldap_con(self):
        if not self._ldapcon:
            self._ldapcon=ldap.ldapobject.ReconnectLDAPObject(ldapurl)
        return self._ldapcon

    def check_ldap_password(self,user,pwd):
        """
            Check password. If OK return the DN of the user and the CN (libelle)
        """

        l=self.ldap_con()
        try:
            l.simple_bind_s()
        except:
            #reinit the connection in case of failure (ldap server restarted ?)
            self._ldapcon=None
            l=self.ldap_con()
            l.simple_bind_s()

        # search for a DN with (uid=user)
        user_dict=l.search_s(ldaproot,ldap.SCOPE_SUBTREE,"(uid=%s)" % user, ['uid','cn'])
        if len(user_dict)==1:
            # Try to bind with the supplied password. If no exception -> password is OK
            ldap_user=ldap.ldapobject.ReconnectLDAPObject(ldapurl)
            ldap_user.simple_bind_s(user_dict[0][0],pwd)
            return (user_dict[0][0],user_dict[0][1]['cn'][0])
        else:
            if len(user_dict)>1:
                raise Exception('Several users found with the same UID')
            else:
                raise ldap.INVALID_CREDENTIALS


    def login(self,user="",pwd="",error="",redirect="index"):
        """
        Login form. Reentrant form.

        """
        LOGINFORM='''
<center>
<form name="f" action="login" method="post"><br>
<input type="hidden" name="redirect" value="%(redirect)s">
<table bgcolor="#ffffff" border="0" width="350">
<tbody>
<tr>
<td align="center" bgcolor="#dcdcdc"><b>SERMO Login</b> </td>
</tr>
<tr>
<td style="vertical-align: top;"><span
style="color: rgb(255, 0, 0); font-weight: bold;">%(error)s</span><br>
</td>
</tr>
<tr>
<td align="left" bgcolor="#ffffff">
<table align="center" bgcolor="#ffffff" border="0" width="100%%">
<tbody>
<tr>
<td align="right" width="30%%">Name:</td>
<td align="left" width="*"><input name="user" value=""
type="text"> </td>
</tr>
<tr>
<td align="right" width="30%%">Password:</td>
<td align="left" width="*"><input name="pwd"
type="password"> </td>
</tr>
</tbody>
</table>
</td>
</tr>
<tr>
<td align="left">
<center><input value="Login" type="submit"></center>
</td>
</tr>
</tbody>
</table>
</form></center>''' % {'error':error,'redirect':redirect}
        if user:
            try:
                (newuserdn,newusercn)=self.check_ldap_password(user,pwd)
                # Found a user in LDAP database
                if newuserdn:
                    self.logout()
                    cherrypy.session['user']=user.upper()
                    cherrypy.session['userdn']=newuserdn
                    cherrypy.session['usercn']=newusercn
                    #print "OK",newuserdn,newusercn
                else:
                    # Error : go back to login form with error
                    raise cherrypy.HTTPRedirect('login?error=%s' % 'user or password incorrect')
                    #return httptools.redirect(self.login)
                #Continue normal processing
            except Exception, e:
                raise cherrypy.HTTPRedirect('login?error=%s' % 'user or password incorrect '+str(e))

            raise cherrypy.HTTPRedirect(redirect)
            #return "Login successful : %s" % cherrypy.session['user']
        else:
            return self.template('Login form',LOGINFORM,'f.user')
    login.exposed=True

    def currentuser(self):
        return cherrypy.session.get('user')

    def currentusercn(self):
        return cherrypy.session.get('usercn')

    def currentuserdn(self):
        return cherrypy.session.get('userdn')

    def usergroups(self):
        if self.currentuser() is None:
            return []
        if cherrypy.session.get('usergroups') is None:
            l=self.ldap_con()
            #l.simple_bind_s()
            # search for (memberUid=currentuser)
            groups=l.search_s(ldaproot,ldap.SCOPE_SUBTREE,"(memberUid=%s)" % self.currentuser(), ['cn'])
            cherrypy.session['usergroups']=[g[1]['cn'][0].upper() for g in groups]
        if cherrypy.session.get('usergroups') is None:
            return []
        else:
            return cherrypy.session.get('usergroups')

    def isingroup(self,agroupname):
        return agroupname.upper() in self.usergroups()

    def ldapuserattributes(self,attnames,uid=''):
        """
            return the dictionary of attributes of current user.
        """
        if not uid and not self.currentuserdn():
            return {}

        if uid:
            dn=self.getdnfromuid(uid)
        else:
            dn=self.currentuserdn()

        l=self.ldap_con()
        try:
            # get attributes from ldap database
            user_dict=l.search_s(dn,ldap.SCOPE_SUBTREE,'(objectClass=*)', attnames)
            # user_dict= [(dn,{att})]
            if user_dict:
                return user_dict[0][1]
            else:
                return {}
        except:
            return {}
    ldapuserattributes.exposed=True

    def userprofile(self):
        atts=self.ldapuserattributes(['homeDirectory','mail'])
        return """
Session Id : %s<br>
User code  : %s<br>
Groups     : %s<br>
Session parameters : %s<br>
ldapconnection : %s<br>
homeDirectory : %s<br>
mails : %s""" % (cherrypy.session['_id'],self.currentuser(),self.usergroups(),cherrypy.session,self.ldap_con().whoami_s(),atts.get('homeDirectory',[''])[0],';'.join(atts.get('mail',[''])))
    userprofile.exposed=True

    def logout(self,redirect=""):
        #cherrypy.session.clear()
        keys=cherrypy.session.keys()
        for att in keys:
            if att<>'_id':
                del cherrypy.session[att]
        """
        cherrypy.session['user']=None
        cherrypy.session['userdn']=None
        cherrypy.session['usercn']=None
        cherrypy.session['usergroups']=None
        """
        if redirect:
            raise cherrypy.HTTPRedirect(redirect)
        else:
            return "Logout successful"
    logout.exposed=True

    def getdnfromuid(self,uid):
    # search for a DN with (uid=user)
        l=self.ldap_con()
        try:
            l.simple_bind_s()
        except:
            #reinit the connection in case of failure (ldap server restarted ?)
            self._ldapcon=None
            l=self.ldap_con()
            l.simple_bind_s()

        user_dict=l.search_s(ldaproot,ldap.SCOPE_SUBTREE,"(uid=%s)" % uid, ['uid','cn'])
        if len(user_dict)==1:
            return user_dict[0][0]
        else:
            if len(user_dict)>1:
                raise Exception('Several users found with the same UID')

    def changepassword(self,error="",redirect="index",oldpwd="",newpwd="",newpwd2=""):
        def form(focus='f.oldpwd'):
            return self.template('Password form',"""
                <center>
                <form name="f" action="changepassword" method="post"><br>
                <input type="hidden" name="redirect" value="%(redirect)s">
                <table bgcolor="#ffffff" border="0" width="350">
                <tbody>
                <tr>
                <td align="center" bgcolor="#dcdcdc"><b>Change password of %(cn)s</b> </td>
                </tr>
                <tr>
                <td style="vertical-align: top;"><span
                style="color: rgb(255, 0, 0); font-weight: bold;">%(error)s</span><br>
                </td>
                </tr>
                <tr>
                <td align="left" bgcolor="#ffffff">
                <table align="center" bgcolor="#ffffff" border="0" width="100%%">
                <tbody>
                <tr>
                <td align="right" width="30%%">Old password:</td>
                <td align="left" width="*"><input name="oldpwd" value="%(oldpwd)s"
                type="password"> </td>
                </tr>
                <tr>
                <td align="right" width="30%%">New Password:</td>
                <td align="left" width="*"><input name="newpwd"
                type="password" value="%(newpwd)s"> </td>
                </tr>
                <tr>
                <td align="right" width="30%%">Repeat new Password:</td>
                <td align="left" width="*"><input name="newpwd2"
                type="password" value="%(newpwd2)s"> </td>
                </tr>
                </tbody>
                </table>
                </td>
                </tr>
                <tr>
                <td align="left">
                <center><input type="submit"></center>
                </td>
                </tr>
                </tbody>
                </table>
                </form></center>""" % {'cn':self.currentusercn(),'error':error,'redirect':redirect,'oldpwd':oldpwd,'newpwd':newpwd,'newpwd2':newpwd2},
                focus)

        if self.currentuser() is None:
            #return Exception("Not logged in")
            return self.login("","","","changepassword")
        if oldpwd:
            if not newpwd:
                error='You must supply a new password'
                return form('f.newpwd')
            if newpwd<>newpwd2:
                error="New passwords don't match"
                return form('f.newpwd2')
            if newpwd and (newpwd==newpwd2):
                try:
                    #Try to authenticate with the supplied old pwd
                    return "pwd changed"
                    #Continue normal processing
                except:
                    error='password is incorrect'
                    return self.template('Password form',FORM,'f.user')
                    #raise cherrypy.HTTPRedirect('login?error=%s' % 'user or password incorrect')
                raise cherrypy.HTTPRedirect(redirect)

            #return "Login successful : %s" % cherrypy.session['user']
        else:
            return form('f.oldpwd')
    changepassword.exposed=True


    def template(self,title,body,focus="f.search"):
        cu=self.currentuser()
        if cu is not None:
            lib=self.currentusercn()
            userinfo="Current user : <a href=userprofile>%s (%s)</a>   <a href=logout?redirect=login>Logout</a>" % (lib,cu)
        else:
            userinfo="Not logged in <a href=login>Login</a>"
        if focus:
            sf="document.%s.focus();"%focus
        else:
            sf=""
        return '''
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=UTF-8">
<title>%(title)s</title>
<link rel="stylesheet" title="Standard" href="http://sermo-sth.intra.sermo.fr/sermo.css" />
<script>
function sf(){%(sf)s}
</script>
</head>
<body onload="sf()">
<div id=userinfo>
%(userinfo)s
</div>
%(body)s
</body>
</html>
            ''' % {'title':title,'body':body,'sf':sf,'userinfo':userinfo}

